# -*- coding:Utf-8 -*-


# ==================================================
# >> IMPORTATIONS
# ==================================================


import es
import gamethread

import os
import re
import random

import psyco
psyco.full()


# ==================================================
# >> INFORMATIONS
# ==================================================


informations = es.AddonInfo()
informations["name"] = "Parachute by L'In20Cible"
informations["version"] = "1.1"
informations["date"] = "2010-05-08"
informations["author"] = "L\'in20Cible"
informations["url"] = "http://addons.eventscripts.com/addons/view/parachute/"
informations["basename"] = "parachute"

es.ServerVar("pc_version", (informations["version"] + " by " + informations["author"])).makepublic()


# ==================================================
# >> GLOBALES
# ==================================================


options = {
   "enable": ("1", "pc_enable <1/0> -- Active/Désactive le parachute."),
   "admins": ("0", "pc_admins <1/0> -- Active/Désactive la restriction du parachute aux joueurs qui ne sont pas administrateurs."),
   "models": ("0", "pc_models <1/0> -- Active/Désactive les apparences du parachute.")
   }
for option in options:
   options[option] = es.ServerVar(("pc_" + option), options[option][0], options[option][1])
   
   
# ==================================================
# >> INITIALISATION
# ==================================================


def load():
   Initialization()
   es.addons.registerTickListener(TickListenerCallBack)
   
   
def Initialization():
   global players, admins, models
   path = GetConfigurationFilePath("configuration.cfg")
   if os.path.isfile(path):
      es.mexec(path.replace(str(es.ServerVar("eventscripts_gamedir")).replace(os.sep, "/"), ".."))
   models = GetConfigurationFileParsed(GetConfigurationFilePath("models.txt"))
   admins = GetConfigurationFileParsed(GetConfigurationFilePath("admins.txt"))
   if not str(es.ServerVar("eventscripts_currentmap")) <> "":
      return
   for line in GetConfigurationFileParsed(GetConfigurationFilePath("downloads.txt")):
      if not os.path.isfile(os.path.join(str(es.ServerVar("eventscripts_gamedir")), line)):
         continue
      es.stringtable("downloadables", line)
   players = dict()
   
def unload():
   es.addons.unregisterTickListener(TickListenerCallBack)
   
   
# ==================================================
# >> UTILITAIRES
# ==================================================


def TickListenerCallBack():
   if not int(options["enable"]):
      return
   for userid in es.getUseridList():
      if es.getplayerprop(userid, "CCSPlayer.pl.deadflag") or (options["admins"] and (es.getplayersteamid(userid) not in admins)):
         continue
      if userid not in players:
         if es.getplayerprop(userid, "CCSPlayer.baseclass.localdata.m_Local.m_flFallVelocity") > 0.0:
            OpenPlayerParachute(userid)
      else:
         if es.getplayerprop(userid, "CCSPlayer.baseclass.localdata.m_hGroundEntity") <> -1:
            ClosePlayerParachute(userid)
         else:
            SetEntityLocation(players[userid], es.getplayerlocation(userid))
            
            
# ==================================================
# >> UTILITAIRES
# ==================================================


def GetConfigurationFilePath(file):
   return os.path.join(es.getAddonPath(informations["basename"]), "configuration", file).replace(os.sep, "/")
   
   
def GetConfigurationFileParsed(path):
   lines = list()
   if not os.path.isfile(path):
      return lines
   regex = re.compile("//.*|^(( |\t)+)|(( |\t)+)$|(\r)+|(\n)+")
   for line in open(path):
      line = regex.sub("", line).strip()
      if not line:
         continue
      lines.append(line)
   return lines
   
   
def OpenPlayerParachute(userid):
   players[userid] = 0
   SetPlayerGravity(userid, "0.1")
   if not int(options["models"]) or not len(models):
      return
   es.server.cmd("es_xprop_physics_create %i props/cs_office/water_bottle.mdl" % userid)
   players[userid] = int(es.ServerVar("eventscripts_lastgive"))
   es.setindexprop(players[userid], "CBaseEntity.m_CollisionGroup", 2)
   SetEntityLocation(players[userid], es.getplayerlocation(userid))
   es.setindexprop(players[userid], "CBaseEntity.movetype", 0)
   es.setindexprop(players[userid], "CBaseEntity.m_nModelIndex", es.precachemodel(random.choice(models)))
   es.setindexprop(players[userid], "CBaseEntity.m_clrRender", -1)
   
   
def ClosePlayerParachute(userid):
   SetPlayerGravity(userid, "1.0")
   if userid not in players:
      return
   if players[userid] in es.createentitylist("prop_physics"):
      es.server.queuecmd(("est_remove " + str(players[userid])))
   del players[userid]
      
      
def SetPlayerGravity(userid, value):
   es.server.queuecmd("es_xfire %i !self AddOutput \"gravity %s\"" % (userid, value))
   
   
def SetEntityLocation(index, origin):
   if index not in es.createentitylist("prop_physics"):
      return
   es.server.cmd("est_entteleport %i %s %s %s" % (index, origin[0], origin[1], origin[2]))
   
   
# ==================================================
# >> ÉVÉNEMENTS
# ==================================================


def es_map_start(event_var):
   Initialization()
   
   
def player_death(event_var):
   ClosePlayerParachute(int(event_var["userid"]))
   
   
def player_disconnect(event_var):
   ClosePlayerParachute(int(event_var["userid"]))